#include <stdlib.h> 

typedef int8_t pos_t;

typedef enum {
	NODIR, NORTH, EAST, SOUTH, WEST
} Direction;

typedef enum {
	RABBIT
} PieceType;

/**
 * @author David Croft (david@davidc.net)
 * @version $Id$
 */
 
 
class SnakePart;
class Piece;

class GameBoard
{
private:
	pos_t width;
	pos_t height;

	SnakePart *snakeTail;
	uint8_t snakeGrow; // number of pieces to grow when we move

public:
	SnakePart *snakeHead;
	Piece *firstPiece;

	Direction snakeFacing;
	uint16_t score;
	uint8_t snakeDead;

	GameBoard(pos_t width, pos_t height);
	~GameBoard();

	void initRabbits();

	void move();
	void moveSnake();
	
	void setFacing(Direction d);
	Direction opposite(Direction d);

	void populateRabbits();
	void addPieceAtRandomUnoccupiedPoint(PieceType type, int minLifetime, int maxLifetime);

	void * operator new(size_t size); 
	void operator delete(void * ptr); 
};


class SnakePart
{
public:
	SnakePart *previous;
	SnakePart *next;
	pos_t x;
	pos_t y;
	Direction prevPieceDir;
	Direction nextPieceDir;

public:
    SnakePart(pos_t x, pos_t y)
	{
		this->x = x;
		this->y = y;
	}

	void * operator new(size_t size); 
	void operator delete(void * ptr); 
};

class Piece
{
public:
	PieceType type;

	pos_t x;
	pos_t y;

	uint8_t lifetimeRemaining;
	
	Piece *next;

	Piece(PieceType type, pos_t x, pos_t y, uint8_t lifetimeRemaining)
	{
		this->type = type;
		this->x = x;
		this->y = y;
		this->lifetimeRemaining = lifetimeRemaining;
		next = 0;
	}

	void * operator new(size_t size); 
	void operator delete(void * ptr); 
};
